"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const traverser_1 = __importDefault(require("eslint/lib/util/traverser"));
const typescriptESTree = __importStar(require("@corbinu/typescript-estree"));
const analyze_scope_1 = require("./analyze-scope");
const visitor_keys_1 = require("./visitor-keys");
const packageJSON = require('../package.json');
exports.version = packageJSON.version;
function parse(code, options) {
    return parseForESLint(code, options).ast;
}
exports.parse = parse;
exports.Syntax = Object.freeze(typescriptESTree.AST_NODE_TYPES);
function parseForESLint(code, options) {
    if (typeof options !== 'object' || options === null) {
        options = { useJSXTextNode: true };
    }
    else if (typeof options.useJSXTextNode !== 'boolean') {
        options = Object.assign({}, options, { useJSXTextNode: true });
    }
    if (typeof options.filePath === 'string') {
        const tsx = options.filePath.endsWith('.tsx');
        if (tsx || options.filePath.endsWith('.ts')) {
            options = Object.assign({}, options, { jsx: tsx });
        }
    }
    // https://eslint.org/docs/user-guide/configuring#specifying-parser-options
    // if sourceType is not provided by default eslint expect that it will be set to "script"
    options.sourceType = options.sourceType || 'script';
    if (options.sourceType !== 'module' && options.sourceType !== 'script') {
        options.sourceType = 'script';
    }
    const { ast, services } = typescriptESTree.parseAndGenerateServices(code, options);
    ast.sourceType = options.sourceType;
    traverser_1.default.traverse(ast, {
        enter: (node) => {
            switch (node.type) {
                // Function#body cannot be null in ESTree spec.
                case 'FunctionExpression':
                    if (!node.body) {
                        node.type = `TSEmptyBody${node.type}`;
                    }
                    break;
                // no default
            }
        }
    });
    const scopeManager = analyze_scope_1.analyzeScope(ast, options);
    return { ast, services, scopeManager, visitorKeys: visitor_keys_1.visitorKeys };
}
exports.parseForESLint = parseForESLint;
